@@ -27,7 +27,7 @@
 <script type="text/javascript">
 	jQuery(function($) {
 		$('form[name="user_form"]').submit(function() {
-			$(this).trimValues(['#password1', '#password2', '#autologout', '#refresh', '#url']);
+			$(this).trimValues(['#autologout', '#refresh', '#url']);
 		});
 
 		$('#messages_enabled').on('change', function() {
