@@ -23,9 +23,6 @@
  * @var CView $this
  */
 
-// Visibility box javascript is already added in main page. It should not be added in popup response.
-define('CVISIBILITYBOX_JAVASCRIPT_INSERTED', 1);
-
 $output = [
 	'header' => $data['title']
 ];
@@ -228,27 +225,11 @@ $operations_popup_form_list
 			->setChecked(array_key_exists('opseverity', $options))
 			->setReadonly($options['templated']),
 		(new CDiv(
-			(new CSeverity([
-				'name' => 'opseverity[severity]',
-				'value' => (int) $field_values['opseverity']['severity']
-			]))->setReadonly($options['templated'])
+			(new CSeverity('opseverity[severity]', (int) $field_values['opseverity']['severity']))
+				->setReadonly($options['templated'])
 		))->setId('opseverity_div'),
 		'opseverity_row'
 	)
-	->addRow(
-		(new CVisibilityBox('visible[optag]', 'optag_div', _('Original')))
-			->setLabel(_('Tags'))
-			->setChecked(array_key_exists('optag', $options))
-			->setReadonly($options['templated']),
-		(new CDiv(
-			renderTagTable($field_values['optag'], $options['templated'],
-					['field_name' => 'optag', 'add_post_js' => false])
-				->setHeader([_('Name'), _('Value'), _('Action')])
-				->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-				->setId('tags-table')
-		))->setId('optag_div'),
-		'optag_row'
-	)
 	->addRow(
 		(new CVisibilityBox('visible[optemplate]', 'optemplate_div', _('Original')))
 			->setLabel(_('Link templates'))
@@ -273,6 +254,20 @@ $operations_popup_form_list
 		]))->setId('optemplate_div'),
 		'optemplate_row'
 	)
+	->addRow(
+		(new CVisibilityBox('visible[optag]', 'optag_div', _('Original')))
+			->setLabel(_('Tags'))
+			->setChecked(array_key_exists('optag', $options))
+			->setReadonly($options['templated']),
+		(new CDiv(
+			renderTagTable($field_values['optag'], $options['templated'],
+					['field_name' => 'optag', 'add_post_js' => false])
+				->setHeader([_('Name'), _('Value'), _('Action')])
+				->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+				->addClass('tags-table')
+		))->setId('optag_div'),
+		'optag_row'
+	)
 	->addRow(
 		(new CVisibilityBox('visible[opinventory]', 'opinventory_inventory_mode', _('Original')))
 			->setLabel(_('Host inventory'))
