@@ -327,7 +327,8 @@ class CSvgGraph extends CSvg {
 			$this->left_y_max = $options['max'];
 		}
 		if ($options['units'] !== null) {
-			$this->left_y_units = trim(preg_replace('/\s+/', ' ', $options['units']));
+			$units = trim(preg_replace('/\s+/', ' ', $options['units']));
+			$this->left_y_units = htmlspecialchars($units);
 		}
 
 		return $this;
@@ -354,7 +355,8 @@ class CSvgGraph extends CSvg {
 			$this->right_y_max = $options['max'];
 		}
 		if ($options['units'] !== null) {
-			$this->right_y_units = trim(preg_replace('/\s+/', ' ', $options['units']));
+			$units = trim(preg_replace('/\s+/', ' ', $options['units']));
+			$this->right_y_units = htmlspecialchars($units);
 		}
 
 		return $this;
@@ -560,8 +562,10 @@ class CSvgGraph extends CSvg {
 
 		$this->left_y_is_binary = $this->left_y_units === 'B' || $this->left_y_units === 'Bps';
 
+		$calc_power = $this->left_y_units === '' || $this->left_y_units[0] !== '!';
+
 		$result = calculateGraphScaleExtremes($this->left_y_min, $this->left_y_max, $this->left_y_is_binary,
-			$this->left_y_min_calculated, $this->left_y_max_calculated, $rows_min, $rows_max
+			$calc_power, $this->left_y_min_calculated, $this->left_y_max_calculated, $rows_min, $rows_max
 		);
 
 		[
@@ -589,8 +593,10 @@ class CSvgGraph extends CSvg {
 
 		$this->right_y_is_binary = $this->right_y_units === 'B' || $this->right_y_units === 'Bps';
 
+		$calc_power = $this->right_y_units === '' || $this->right_y_units[0] !== '!';
+
 		$result = calculateGraphScaleExtremes($this->right_y_min, $this->right_y_max, $this->right_y_is_binary,
-			$this->right_y_min_calculated, $this->right_y_max_calculated, $rows_min, $rows_max
+			$calc_power, $this->right_y_min_calculated, $this->right_y_max_calculated, $rows_min, $rows_max
 		);
 
 		[
@@ -1110,7 +1116,7 @@ class CSvgGraph extends CSvg {
 					->setArgument('eventid', $problem['eventid'])
 					->getUrl(),
 				'r_eventid' => $problem['r_eventid'],
-				'severity' => getSeverityStyle($problem['severity'], $problem['r_clock'] == 0),
+				'severity' => CSeverityHelper::getStyle((int) $problem['severity'], $problem['r_clock'] == 0),
 				'status' => $status_str,
 				'status_color' => $status_color
 			];
