@@ -190,7 +190,6 @@ function checkAll(form_name, chkMain, shkName) {
 
 	chkbxRange.checkObjectAll(shkName, value);
 	chkbxRange.update(shkName);
-	chkbxRange.saveSessionStorage(shkName);
 
 	return true;
 }
@@ -234,7 +233,7 @@ function removeVarsBySelector(form_name, selector) {
 function create_var(form_name, var_name, var_value, doSubmit) {
 	var objForm = is_string(form_name) ? document.forms[form_name] : form_name;
 	if (!objForm) {
-		return false;
+		return;
 	}
 
 	var objVar = (typeof(objForm[var_name]) != 'undefined') ? objForm[var_name] : null;
@@ -242,7 +241,7 @@ function create_var(form_name, var_name, var_value, doSubmit) {
 		objVar = document.createElement('input');
 		objVar.setAttribute('type', 'hidden');
 		if (!objVar) {
-			return false;
+			return;
 		}
 		objVar.setAttribute('name', var_name);
 		objVar.setAttribute('id', var_name.replace(']', '').replace('[', '_'));
@@ -259,8 +258,6 @@ function create_var(form_name, var_name, var_value, doSubmit) {
 	if (doSubmit) {
 		objForm.submit();
 	}
-
-	return false;
 }
 
 function getDimensions(obj) {
@@ -326,47 +323,36 @@ function getPosition(obj) {
 /**
  * Opens popup content in overlay dialogue.
  *
- * @param {string} action         Popup controller related action.
- * @param {array}  options        (optional) Array with key/value pairs that will be used as query for popup request.
- * @param {string} dialogueid     (optional) id of overlay dialogue.
- * @param {object} trigger_elmnt  (optional) UI element which was clicked to open overlay dialogue.
+ * @param {string}           action           Popup controller related action.
+ * @param {array|object}     parameters       Array with key/value pairs that will be used as query for popup request.
+ *
+ * @param {string}           dialogue_class   CSS class, usually based on .modal-popup and .modal-popup-{size}.
+ * @param {string|null}      dialogueid       ID of overlay dialogue.
+ * @param {HTMLElement|null} trigger_element  UI element which was clicked to open overlay dialogue.
  *
  * @returns {Overlay}
  */
-function PopUp(action, options, dialogueid, trigger_elmnt) {
+function PopUp(action, parameters, {
+	dialogueid = null,
+	dialogue_class = '',
+	trigger_element = document.activeElement
+} = {}) {
 	var overlay = overlays_stack.getById(dialogueid);
+
 	if (!overlay) {
-		var wide_popup_actions = ['popup.generic', 'popup.scriptexec', 'dashboard.share.edit',
-				'dashboard.properties.edit', 'popup.services', 'popup.media', 'popup.lldoperation', 'popup.lldoverride',
-				'popup.preproctest.edit', 'popup.triggerexpr', 'popup.httpstep', 'popup.testtriggerexpr',
-				'popup.triggerwizard'
-			],
-			medium_popup_actions = ['popup.maintenance.period', 'popup.condition.actions', 'popup.condition.operations',
-				'popup.condition.event.corr', 'popup.discovery.check', 'popup.mediatypetest.edit',
-				'popup.mediatype.message'
-			],
-			dialogue_class = '';
-
-		if (wide_popup_actions.indexOf(action) !== -1) {
-			dialogue_class = ' modal-popup-generic';
-		}
-		else if (medium_popup_actions.indexOf(action) !== -1) {
-			dialogue_class = ' modal-popup-medium';
-		}
-
 		overlay = overlayDialogue({
-			'dialogueid': dialogueid,
-			'title': '',
-			'content': jQuery('<div>', {'height': '68px', class: 'is-loading'}),
-			'class': 'modal-popup' + dialogue_class,
-			'buttons': [],
-			'element': trigger_elmnt,
-			'type': 'popup'
+			dialogueid,
+			title: '',
+			content: jQuery('<div>', {'height': '68px', class: 'is-loading'}),
+			class: 'modal-popup ' + dialogue_class,
+			buttons: [],
+			element: trigger_element,
+			type: 'popup'
 		});
 	}
 
 	overlay
-		.load(action, options)
+		.load(action, parameters)
 		.then(function(resp) {
 			if (typeof resp.errors !== 'undefined') {
 				overlay.setProperties({
@@ -376,12 +362,25 @@ function PopUp(action, options, dialogueid, trigger_elmnt) {
 			else {
 				var buttons = resp.buttons !== null ? resp.buttons : [];
 
-				buttons.push({
-					'title': t('Cancel'),
-					'class': 'btn-alt',
-					'cancel': true,
-					'action': (typeof resp.cancel_action !== 'undefined') ? resp.cancel_action : function() {}
-				});
+				switch (action) {
+					case 'popup.scheduledreport.list':
+					case 'popup.scheduledreport.test':
+					case 'popup.scriptexec':
+						buttons.push({
+							'title': t('Ok'),
+							'cancel': true,
+							'action': (typeof resp.cancel_action !== 'undefined') ? resp.cancel_action : function() {}
+						});
+						break;
+
+					default:
+						buttons.push({
+							'title': t('Cancel'),
+							'class': 'btn-alt js-cancel',
+							'cancel': true,
+							'action': (typeof resp.cancel_action !== 'undefined') ? resp.cancel_action : function() {}
+						});
+				}
 
 				overlay.setProperties({
 					title: resp.header,
@@ -406,35 +405,30 @@ function PopUp(action, options, dialogueid, trigger_elmnt) {
 /**
  * Open "Update problem" dialog and manage URL change.
  *
- * @param {array}  options
- * @param {array}  options['eventids']  Eventids to update.
- * @param {object} trigger_elmnt        (optional) UI element which was clicked to open overlay dialogue.
+ * @param {Object} parameters
+ * @param {array}  parameters['eventids']  Eventids to update.
+ * @param {object} trigger_element        (optional) UI element which was clicked to open overlay dialogue.
  *
  * @returns {Overlay}
  */
-function acknowledgePopUp(options, trigger_elmnt) {
-	var overlay = PopUp('popup.acknowledge.edit', options, null, trigger_elmnt),
+function acknowledgePopUp(parameters, trigger_element) {
+	var overlay = PopUp('popup.acknowledge.edit', parameters, {trigger_element}),
 		backurl = location.href;
 
-	overlay.trigger_parents = $(trigger_elmnt).parents();
+	overlay.trigger_parents = $(trigger_element).parents();
 
 	overlay.xhr.then(function() {
 		var url = new Curl('zabbix.php', false);
 		url.setArgument('action', 'popup');
 		url.setArgument('popup_action', 'acknowledge.edit');
-		url.setArgument('eventids', options.eventids);
+		url.setArgument('eventids', parameters.eventids);
 
 		history.replaceState({}, '', url.getUrl());
 	});
 
-	var close = function(e, dialogue) {
-		if (dialogue.dialogueid === overlay.dialogueid) {
-			history.replaceState({}, '', backurl);
-			$.unsubscribe('overlay.close', close);
-		}
-	};
-
-	$.subscribe('overlay.close', close);
+	overlay.$dialogue[0].addEventListener('overlay.close', () => {
+		history.replaceState({}, '', backurl);
+	}, {once: true});
 
 	return overlay;
 }
@@ -469,11 +463,6 @@ function addToOverlaysStack(id, element, type, xhr) {
 // Keydown handler. Closes last opened overlay UI element.
 function closeDialogHandler(event) {
 	if (event.which == 27) { // ESC
-		// Do not catch multiselect events.
-		if (jQuery(event.target).closest('.multiselect').data('multiSelect') !== undefined) {
-			return;
-		}
-
 		var dialog = overlays_stack.end();
 		if (typeof dialog !== 'undefined') {
 			switch (dialog.type) {
@@ -527,8 +516,7 @@ function closeDialogHandler(event) {
  * @return {object|undefined|null}  Overlay object, if found.
  */
 function removeFromOverlaysStack(dialogueid, return_focus) {
-	var overlay = null,
-		index;
+	var overlay = null;
 
 	if (return_focus !== false) {
 		return_focus = true;
@@ -554,15 +542,16 @@ function removeFromOverlaysStack(dialogueid, return_focus) {
  * @param {string} action	(optional) action value that is used in CRouter. Default value is 'popup.generic'.
  */
 function reloadPopup(form, action) {
-	var dialogueid = jQuery(form).closest('[data-dialogueid]').attr('data-dialogueid'),
+	var dialogueid = form.closest('[data-dialogueid]').dataset.dialogueid,
+		dialogue_class = jQuery(form).closest('[data-dialogueid]').prop('class'),
 		action = action || 'popup.generic',
-		options = {};
+		parameters = {};
 
-	jQuery(form.elements).each(function() {
-		options[this.name] = this.value;
-	});
+	for (const input of form.elements) {
+		parameters[input.name] = input.value;
+	};
 
-	PopUp(action, options, dialogueid);
+	PopUp(action, parameters, {dialogueid, dialogue_class});
 }
 
 /**
@@ -620,7 +609,7 @@ function addValues(frame, values, submit_parent) {
 		}
 
 		if (jQuery(frm_storage).is('span')) {
-			jQuery(frm_storage).text(values[key]);
+			jQuery(frm_storage).html(values[key]);
 		}
 		else {
 			jQuery(frm_storage).val(values[key]).change();
@@ -670,7 +659,7 @@ function addSelectedValues(object, parentid) {
  */
 function add_media(formname, media, mediatypeid, sendto, period, active, severity) {
 	var form = window.document.forms[formname];
-	var media_name = (media > -1) ? 'user_medias[' + media + ']' : 'new_media';
+	var media_name = (media > -1) ? 'medias[' + media + ']' : 'new_media';
 
 	window.create_var(form, media_name + '[mediatypeid]', mediatypeid);
 	if (typeof sendto === "object") {
@@ -798,6 +787,63 @@ function redirect(uri, method, needle, invert_needle, add_sid, allow_empty) {
 	return false;
 }
 
+/**
+ * Send parameters to given url using natural HTML form submission.
+ *
+ * @param {string} url
+ * @param {Object} params
+ * @param {boolean} allow_empty
+ *
+ * @return {boolean}
+ */
+function post(url, params) {
+	function addVars(post_form, name, value) {
+		if (Array.isArray(value)) {
+			for (let i = 0; i < value.length; i++) {
+				addVars(post_form, `${name}[]`, value[i]);
+			}
+		}
+		else if (typeof value === 'object' && value !== null) {
+			for (const [key, _value] of Object.entries(value)) {
+				addVars(post_form, `${name}[${key}]`, _value);
+			}
+		}
+		else {
+			addVar(post_form, name, value);
+		}
+	}
+
+	function addVar(post_form, name, value) {
+		const is_multiline = /\r|\n/.exec(value);
+		let input;
+
+		if (is_multiline) {
+			input = document.createElement('textarea');
+		}
+		else {
+			input = document.createElement('input');
+			input.type = 'hidden';
+		}
+
+		input.name = name;
+		input.value = value;
+		post_form.appendChild(input);
+	}
+
+	const dom_body = document.getElementsByTagName('body')[0];
+	const post_form = document.createElement('form');
+	post_form.setAttribute('action', url);
+	post_form.setAttribute('method', 'post');
+	post_form.style.display = 'none';
+
+	for (const [key, value] of Object.entries(params)) {
+		addVars(post_form, key, value);
+	}
+
+	dom_body.appendChild(post_form);
+	post_form.submit();
+}
+
 function showHide(obj) {
 	if (jQuery(obj).is(':hidden')) {
 		jQuery(obj).css('display', 'block');
@@ -882,6 +928,25 @@ function appendZero(val) {
 	return val < 10 ? '0' + val : val;
 }
 
+/**
+ * Function converts unix timestamp to human readable time in format 'Y-m-d H:i:s'.
+ *
+ * @param {type} time   Unix timestamp to convert.
+ *
+ * @returns {string}
+ */
+function time2str(time) {
+	var dt = new Date(time * 1000),
+		Y = dt.getFullYear(),
+		m = appendZero(dt.getMonth()+1),
+		d = appendZero(dt.getDate()),
+		H = appendZero(dt.getHours()),
+		i = appendZero(dt.getMinutes()),
+		s = appendZero(dt.getSeconds());
+
+	return Y + '-' + m + '-' + d + ' ' + H + ':' + i + ':' + s;
+}
+
 /**
  * Trims selected element values.
  *
@@ -934,29 +999,100 @@ Function.prototype.bindAsEventListener = function (context) {
 	};
 };
 
-/**
- * Get first selected host from multiselect field.
- *
- * @param {string} host_field_id       Host field element ID.
- * @param {string} hostgroup_field_id  Host group field element ID.
- *
- * @return {object}
- */
-function getFirstMultiselectValue(host_field_id, hostgroup_field_id) {
-	var host_values = (typeof host_field_id !== 'undefined')
-			? jQuery('#'+host_field_id).multiSelect('getData')
-			: [],
-		hostgroup_values = (typeof hostgroup_field_id !== 'undefined')
-			? jQuery('#'+hostgroup_field_id).multiSelect('getData')
-			: [],
-		ret = {};
+function openMassupdatePopup(action, parameters = {}, {
+	dialogue_class = '',
+	trigger_element = document.activeElement
+}) {
+	const form = trigger_element.closest('form');
 
-	if (host_values.length != 0) {
-		ret.hostid = host_values[0].id;
-	}
-	if (hostgroup_values.length != 0) {
-		ret.groupid = hostgroup_values[0].id;
+	switch (action) {
+		case 'popup.massupdate.host':
+			parameters.hostids = chkbxRange.getSelectedIds();
+			break;
+
+		default:
+			parameters.ids = chkbxRange.getSelectedIds();
 	}
 
-	return ret;
+	switch (action) {
+		case 'popup.massupdate.item':
+			parameters.context = form.querySelector('#form_context').value;
+			parameters.prototype = 0;
+			break;
+
+		case 'popup.massupdate.trigger':
+			parameters.context = form.querySelector('#form_context').value;
+			break;
+
+		case 'popup.massupdate.itemprototype':
+		case 'popup.massupdate.triggerprototype':
+			parameters.parent_discoveryid = form.querySelector('#form_parent_discoveryid').value;
+			parameters.context = form.querySelector('#form_context').value;
+			parameters.prototype = 1;
+			break;
+	}
+
+	return PopUp(action, parameters, {dialogue_class, trigger_element});
+}
+
+/**
+ * @param {boolean} value
+ * @param {string} objectid
+ * @param {string} replace_to
+ */
+function visibilityStatusChanges(value, objectid, replace_to) {
+	const obj = document.getElementById(objectid);
+
+	if (obj === null) {
+		throw `Cannot find objects with name [${objectid}]`;
+	}
+
+	if (replace_to && replace_to != '') {
+		if (obj.originalObject) {
+			const old_obj = obj.originalObject;
+			old_obj.originalObject = obj;
+
+			obj.parentNode.replaceChild(old_obj, obj);
+		}
+		else if (!value) {
+			const new_obj = document.createElement('span');
+			new_obj.setAttribute('name', obj.name);
+			new_obj.setAttribute('id', obj.id);
+			new_obj.innerHTML = replace_to;
+			new_obj.originalObject = obj;
+
+			obj.parentNode.replaceChild(new_obj, obj);
+		}
+		else {
+			throw 'Missing originalObject for restoring';
+		}
+	}
+	else {
+		obj.style.visibility = value ? 'visible' : 'hidden';
+	}
+}
+
+/**
+ * Clears session storage from markers of checked table rows.
+ * Or keeps only accessible IDs in the list of checked rows.
+ *
+ * @param {string}       page
+ * @param {array|Object} keepids
+ */
+function uncheckTableRows(page, keepids = []) {
+	// This key only works for new MVC pages.
+	const key = (page === '') ? 'cb_zabbix' : 'cb_zabbix_'+page;
+
+	if (keepids.length) {
+		// If keepids will not have same key as value, it will create mess, when new checkbox will be checked.
+		let keepids_formatted = {};
+		for (const id of Object.values(keepids)) {
+			keepids_formatted[id.toString()] = id.toString();
+		}
+
+		sessionStorage.setItem(key, JSON.stringify(keepids_formatted));
+	}
+	else {
+		sessionStorage.removeItem(key);
+	}
 }
